/*
 * abissd/sched_basic.c - abissd, "basic" scheduler
 *
 * Written 2004 by Werner Almesberger
 *
 * (C) Copyright Koninklijke Philips Electronics NV 2004. All rights reserved.
 * For licensing and warranty information, see the file ../../COPYING.GPLv2
 */


#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <abiss_msg_basic.h>
#include <abissd.h>


extern struct sched_ops sched_ops;


static void init_vars(void)
{
    var_register(&sched_ops,"batch_bytes",'i',"read batch size");
}


static void print_params(struct abissd_open_msg *msg)
{
    struct abiss_sched_basic_prm prm;

    fprintf(stderr,"  (expecting %d)\n",
      (int) sizeof(struct abiss_sched_basic_prm));

    memcpy(&prm,msg+1,sizeof(prm));

    fprintf(stderr,"  ra_bytes       %d\n",(int) prm.ra_bytes);
    fprintf(stderr,"  batch_bytes    %d\n",(int) prm.batch_bytes);
    fprintf(stderr,"  fill_Bps       %d\n",prm.fill_Bps);
    fprintf(stderr,"  consider_write %d\n",(int) prm.consider_write);
}


static int open_stream(struct stream *stream,struct abissd_open_msg *msg)
{
    struct abiss_sched_basic_prm *_prm =
      (struct abiss_sched_basic_prm *) (msg+1);
    struct abiss_sched_basic_prm prm;

    memcpy(&prm,_prm,sizeof(prm));

    stream->rm.byterate = prm.fill_Bps;
    stream->rm.bufsize = prm.ra_bytes;
    if (var_is_set("batch_bytes"))
	prm.batch_bytes = var_get_int("batch_bytes");

    if (verbose)
	print_params(msg);
    
    memcpy(_prm,&prm,sizeof(prm));
    return rm_reserve(&stream->rm) ? 0 : -EDQUOT;
}


static void close_stream(struct stream *stream)
{
    rm_return(&stream->rm);
}


struct sched_ops sched_ops = {
    .name = "basic",
    .init = init_vars,
    .print_params = print_params,
    .open = open_stream,
    .close = close_stream,
};

PLUGIN_LICENSE("GPLv2");
