/*
 * abissd/open.c - abissd, "open" message
 *
 * Written 2004 by Werner Almesberger
 *
 * (C) Copyright Koninklijke Philips Electronics NV 2004. All rights reserved.
 * For licensing and warranty information, see the file ../../COPYING.LGPLv21
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <errno.h>

#include <abissd.h>

#include "common.h"
#include "rm.h"
#include "stream.h"


int fixed_reply = 0;
int replies[100]; /* that should be enough for a while */
int *reply = replies;
static int *this_reply = replies;


size_t abissd_open(void *buffer)
{
    struct abissd_open_msg *msg = buffer;
    const struct abissd_extent *extents;
    const struct sched_ops *sched_ops;
    struct stream *stream;

    sched_ops = find_sched(msg->sched_name);
    if (!sched_ops) {
	fprintf(stderr,"don't know scheduler \"%s\"\n",msg->sched_name);
	msg->error = -EINVAL;
	return sizeof(*msg);
    }

    extents =
      (struct abissd_extent *) (buffer+sizeof(*msg)+msg->sched_prm_len);

    if (verbose > 1) {
	int i;
	fprintf(stderr,"  id: dev 0x%llx, ino %llu, fid 0x%llx\n",
	  (unsigned long long) msg->id.dev,
	  (unsigned long long) msg->id.ino,
	  (unsigned long long) msg->id.fid);
	    fprintf(stderr,"  mode %d (%s)\n",msg->mode,
		msg->mode == O_RDONLY ? "O_RDONLY" :
		  msg->mode == O_WRONLY ? "O_WRONLY" :
		    msg->mode == O_RDWR ? "O_RDWR " : "???");
	fprintf(stderr,"  PID %d\n",msg->pid);
	fprintf(stderr,"  scheduler \"%s\"\n",msg->sched_name);
	fprintf(stderr,"  %d bytes of scheduler parameters:\n",
	  msg->sched_prm_len);

	if (sched_ops->print_params)
		sched_ops->print_params(msg);

	fprintf(stderr,"  size: %lld bytes\n",(long long) msg->file_size);
	fprintf(stderr,"  %d extents:\n",msg->num_extents);

	for (i = 0; i != msg->num_extents; i++) {
	    struct abissd_extent extent;

	    memcpy(&extent,extents+i,sizeof(extent));
	    fprintf(stderr,"  [%d] %llu+%llu\n",i,
	      (unsigned long long) extent.start,
	      (unsigned long long) extent.length);
	}
    }

    if (fixed_reply) {  
 	msg->error = *this_reply;
	if (this_reply+1 != reply)
	    this_reply++;
	return (sizeof(*msg)+msg->sched_prm_len);
    }

    msg->error = 0;

    // check if it is a reading request 
    if ((msg->mode != O_RDONLY) && (msg->mode != O_RDWR)) {
	static struct sched_ops null_ops = { .name = "null" };

	// TODO: add write msg
	fprintf(stderr,"  write request: always allowed\n");
	new_stream(&msg->id)->sched_ops = &null_ops;
	return (sizeof(*msg)+msg->sched_prm_len);
    }

    stream = new_stream(&msg->id);
    stream->sched_ops = sched_ops;
    rm_set_extents(&stream->rm,extents,msg->num_extents);
    if (sched_ops->open) {
	msg->error = sched_ops->open(stream,msg);
	if (msg->error)
	    destroy_stream(stream);
    }

    return (sizeof(*msg)+msg->sched_prm_len);
}
