/*
 * abissd/rm.c - abissd, resource management
 *
 * Written 2004 by Werner Almesberger
 *
 * (C) Copyright Koninklijke Philips Electronics NV 2004. All rights reserved.
 * For licensing and warranty information, see the file ../../COPYING.LGPLv21
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>

#include "common.h"
#include "rm.h"


#define MAX_ALLOWED_NUM_FILES	10		// somewhat arbitrary limit
#define MAX_ACCU_BYTERATE	(int)(15e6)	// determined by disk
						// parameters
#define MAX_TOTAL_BUFSIZE	(int)100e6	// determined by available
						// memory


int max_allowed_num_files  = MAX_ALLOWED_NUM_FILES;
int max_accu_byterate      = MAX_ACCU_BYTERATE;
int max_total_bufsize      = MAX_TOTAL_BUFSIZE;

static int num_files = 0;
static int accu_byterate = 0;
static int total_bufsize = 0;


void rm_init(void)
{
}


void rm_set_extents(struct rm *rm,const struct abissd_extent *extents,
  int num_extents)
{
    size_t extents_size;

    rm->num_extents = num_extents;
    extents_size = sizeof(struct abissd_extent)*num_extents;
    rm->extents = malloc(extents_size);
    if (!rm->extents) {
	perror("malloc");
	exit(1);
    }
    memcpy(rm->extents,extents,extents_size);
}


int rm_reserve(struct rm *rm)
{
    if (num_files == max_allowed_num_files) {
	fprintf(stderr,"  too many (%d) files open\n",num_files);
	return 0;
    }
    if (accu_byterate+rm->byterate > max_accu_byterate) {
	fprintf(stderr,"  rate exceeds threshold (%d+%d > %d)\n",
	  accu_byterate,rm->byterate,max_accu_byterate);
	return 0;
    }
    if (total_bufsize+rm->bufsize > max_total_bufsize) {
	fprintf(stderr,"  no bufferspace available (%d+%d > %d)\n",
	  total_bufsize,rm->bufsize,max_total_bufsize);
	return 0;
    }

    num_files++;
    accu_byterate += rm->byterate;
    total_bufsize += rm->bufsize;

    if (verbose > 1)
	fprintf(stderr,"rm_reserve: %d/%d files, %d/%d Bps, %d/%d B\n",
	  num_files,max_allowed_num_files,accu_byterate,max_accu_byterate,
	  total_bufsize,max_total_bufsize);

    return 1;
}


void rm_return(const struct rm *rm)
{
    num_files--;
    accu_byterate -= rm->byterate;
    total_bufsize -= rm->bufsize;

    if (verbose > 1)
	fprintf(stderr,"rm_return: %d/%d files, %d/%d Bps, %d/%d B\n",
	  num_files,max_allowed_num_files,accu_byterate,max_accu_byterate,
	  total_bufsize,max_total_bufsize);

    free(rm->extents);
}
