/*
 * tests/elv/Kernel.h - Fake kernel environment
 *
 * Written 2004 by Werner Almesberger
 *
 * (C) Copyright Koninklijke Philips Electronics NV 2004. All rights reserved.
 * For licensing and warranty information, see the file ../../COPYING.GPLv2
 *
 * Some items copied from kernel files have other/additional copyright holders.
 */

/*
 * #include this only in "kernel" files. It won't pull in any user-space
 * includes.
 */

#ifndef kERNEL_H
#define kERNEL_H

#include "Kglue.h"


/* ----- General stuff ----------------------------------------------------- */


#define NULL 0

typedef typeof(sizeof(int)) size_t;
typedef int ssize_t;

#define isspace(c) 0	/* we don't actually use this */


/* ----- Misc. kernel ------------------------------------------------------ */


#define __setup_file2(str,fn,file) \
    void __setup_fn_##file(char *s) { fn(s); }
#define __setup_file(str,fn,file) __setup_file2(str,fn,file)
#define __setup(str,fn) __setup_file(str,fn,__FILE_NAME__) /* mark as "used" */
#define __init __attribute__((unused))
#define __exit __attribute__((unused))
#define device_initcall(fn)
#define module_init(fn)	void __init_fn(void) { fn(); }
#define module_exit(fn)
#define try_module_get(m) 1
#define module_put(m)

#define likely(x)	(x)
#define unlikely(x)	(x)
#define prefetch(x)	0
#define PageAnon(p)	0

#define EXPORT_SYMBOL(name)
#define EXPORT_SYMBOL_GPL(name)

#define smp_wmb()

typedef struct { int lock; } spinlock_t;
#define spinlock_t spinlock_t __attribute__((unused))
#define SPIN_LOCK_UNLOCKED { 0 }
#define spin_lock_irqsave(lock,flags) do { flags = 1; } while (0)
#define spin_unlock_irqrestore(lock,flags)
#define spin_lock(lock)
#define spin_unlock(lock)
#define spin_lock_irq(lock)
#define spin_unlock_irq(lock)

#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)

#define container_of(ptr, type, member) ({			\
	const typeof( ((type *)0)->member ) *__mptr = (ptr);	\
	(type *)( (char *)__mptr - offsetof(type,member) );})

#define min(x,y) ({ \
	typeof(x) _x = (x);	\
	typeof(y) _y = (y);	\
	(void) (&_x == &_y);	\
	_x < _y ? _x : _y; })

struct kobject {
};

struct attribute {
	const char *name;
	enum { /* looks odd, but makes it clear what we need S_* for */
		S_IRUGO,
		S_IWUSR,
	} mode;
};

struct sysfs_ops {
	ssize_t (*show)(struct kobject *kobj,struct attribute *attr,
	    char *page);
	ssize_t (*store)(struct kobject *kobj,struct attribute *attr,
	    const char *page,size_t count);
};

struct kobj_type {
	struct sysfs_ops *sysfs_ops;
	struct attribute **default_attrs;
};

#define kobject_get(kobj)

#define time_after(a,b)	0	/* 2.6.5 */
#define time_before_eq(a,b)	1

#define jiffies	0
#define HZ	100


/* ---- test_bit ----------------------------------------------------------- */


#define test_bit_QUEUE_FLAG_DRAIN 0
#define test_bit(bit,arg) test_bit_##bit


/* ---- Substituted functions ---------------------------------------------- */


void *kmalloc(size_t size,int gfp_mask);
void kfree(void *ptr);
long simple_strtol(const char *p,char **end,int base);
unsigned long simple_strtoul(const char *p,char **end,int base);
int scnprintf(char *buf,size_t size,const char *fmt,...);
void BUG(void);
#define BUG_ON(cond) do { if (cond) BUG(); } while (0)
#define WARN_ON(cond) do { if (cond) BUG(); } while (0)
    /* we take warnings seriously */

void *memset(void *s,int c,size_t n);
int strcmp(const char *s1,const char *s2);
int strncmp(const char *s1,const char *s2,size_t len);
int strcpy(char *dest,const char *src);
int strncpy(char *dest,const char *src,size_t n);
size_t strlen (const char *s);
void *memcpy(void *d,const void *,size_t n);

extern int no_printk; /* set to non-zero to silence printk */

int printk(const char *fmt,...);
void panic(const char *fmt,...);

int sprintf(char *s,const char *fmt,...);


/* ----- Errors ------------------------------------------------------------ */


enum {
	NO_ERROR,
	ENOMEM,
	EBUSY,
	EWOULDBLOCK,
	ENOTBLK,
	EINVAL,
};


/* ----- Other includes ---------------------------------------------------- */


#define __KERNEL__ /* stop the yapping */
#include "_list.h"
#include "_rbtree.h"
#include "_rbwalk.h"


/* ----- Priority search trees --------------------------------------------- */


#define BITS_PER_LONG (sizeof(long)*8)
#define ULONG_MAX ((((1UL << (BITS_PER_LONG-1))-1) << 1)+1)
#define ARRAY_SIZE(a) (sizeof(a)/sizeof(*(a)))

typedef unsigned long pgoff_t;

#include "_prio_tree.h"

void prio_tree_init(void);


/* ----- MM ---------------------------------------------------------------- */


enum {
	GFP_KERNEL,
	GFP_ATOMIC,
};

struct address_space {
	struct inode *host;
};

struct page {
	struct address_space *mapping;
};


/* ----- IO ---------------------------------------------------------------- */


#define CONFIG_IOSCHED_NOOP

typedef unsigned long sector_t;

#define hard_nr_sectors nr_sectors

struct gendisk {
	int in_flight;
};

struct request {
	int flags;
	struct list_head queuelist;
	void *elevator_private;
	sector_t sector;
	unsigned long nr_sectors;
	struct bio *bio;
	struct bio *biotail;
	struct request_queue *q;
	struct request_list *rl;
	struct gendisk *rq_disk;
	int waiting;		/* must be zero */
	int special;		/* must be zero */
	int data_len;		/* must be zero; 2.6.5 */
	int start_time;		/* must be zero; 2.6.5 */
};

#define list_entry_rq(ptr) list_entry((ptr), struct request, queuelist)

struct request_list {
	int count[2];
};

typedef struct request_queue {
	struct list_head queue_head;
	struct elevator_queue *elevator;
	struct request *last_merge;
	struct request_list rq;
	int (*prep_rq_fn)(struct request_queue *q,struct request *rq);
	int (*merge_requests_fn)(struct request_queue *,struct request *,
                                 struct request *);
	int in_flight;		/* must be zero; 2.6.6 */
	int unplug_thresh;
} request_queue_t;

void abort(void);
#define drain_list queue_head+({abort(); 0;})	/* crash if trying to use it */

#define RQ_NOMERGE_FLAGS        \
        (REQ_NOMERGE | REQ_STARTED | REQ_HARDBARRIER | REQ_SOFTBARRIER)
#define rq_mergeable(rq)        \
        (!((rq)->flags & RQ_NOMERGE_FLAGS))


struct bio_vec {
	struct page *bv_page;
	unsigned int bv_len;
	unsigned int bv_offset;
};

struct bio {
	int bi_rw;
	int bi_vcnt;
	struct bio_vec *bi_io_vec;
	struct bio *bi_next;
	sector_t bi_sector;
	unsigned long bi_size; /* bytes */
	struct block_device *bi_bdev;
	int bi_prio;	/* fake - kernel puts this into bi_rw */
};

enum {
	BLKPREP_OK,
	BLKPREP_DEFER,
	BLKPREP_KILL,
};

struct block_device {
	struct request_queue *q;
	struct gendisk *bd_disk;
};

struct inode {
	int i_abiss;		/* just a flag ... */
};

struct super_block {
	struct block_device *s_bdev;
};

#define bdev_get_queue(bdev) ((bdev)->q)
#define blk_pm_request(rq) \
	((rq)->flags & (REQ_PM_SUSPEND | REQ_PM_RESUME))
#define blk_account_rq(rq) 0
#define bio_sectors(bio) ((bio)->bi_size >> 9)
#define blk_plug_device(q)
#define end_that_request_first(rq,duh,secs)	0
#define end_that_request_last(rq)
#define end_that_request_chunk(rq,duh,bytes) /* 2.6.5 */
#define disk_round_stats(dsk)

#define bio_data_dir(bio) ((bio)->bi_rw & 1)
#define rq_data_dir(rq) ((rq)->flags & 1)

#define bio_set_prio(bio,prio) do { (bio)->bi_prio = (prio); } while (0)
#define bio_prio(bio) ((bio)->bi_prio)

void blkdev_dequeue_request(struct request *rq);

#define blk_queue_plugged(q) 0
#define __generic_unplug_device(q)


/* ----- Elevator ---------------------------------------------------------- */


typedef struct elevator_queue elevator_t;

#include "elevator.h"


/* ----- Diagnostics ------------------------------------------------------- */


#define KERN_DEBUG
#define KERN_ERR
#define KERN_INFO
#define KERN_CRIT
#define KERN_EMERG

#define DPRINTK(...) \
    do { if (debug_level) printk(__VA_ARGS__); } while (0)
#define DPRINTK2(...) \
    do { if (debug_level >= 2) printk(__VA_ARGS__); } while (0)
#define DPRINTK3(...) \
    do { if (debug_level >= 3) printk(__VA_ARGS__); } while (0)
#define DPRINTK4(...) \
    do { if (debug_level >= 4) printk(__VA_ARGS__); } while (0)

#define CONFIG_ABISS_DEBUG_LEVEL debug_level


/* ----- ABISS ------------------------------------------------------------- */


#define ABISS_PRIO_MAX	7

#define abiss_pm_active(ops,ctx,rq,prio)
#define abiss_pm_inactive(ops,ctx)
#define abiss_pm_init(ops,q) NULL
#define abiss_pm_exit(ops,ctx)
#define abiss_pm_frozen(ops,ctx) 0

void elevator_upgrade(const struct super_block *sb,sector_t sector,
    unsigned long nr_sectors,int prio);

#define CONFIG_ABISS

#endif /* kERNEL_H */
