/*
 * Script.c - C framework for RPST unit tests
 *
 * Written 2004, 2005 by Werner Almesberger
 *
 * (C) Copyright Koninklijke Philips Electronics NV 2004, 2005. All rights
 * reserved.
 * For licensing and warranty information, see the file ../../COPYING.GPLv2
 */


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>


#define BUF_SIZE 20000


/*
 * Various minor items used by prio_tree
 */

typedef unsigned long pgoff_t;
#define BITS_PER_LONG (sizeof(long)*8)
#define ULONG_MAX ((((1UL << (BITS_PER_LONG-1))-1) << 1)+1)
#define ARRAY_SIZE(a) (sizeof(a)/sizeof(*(a)))
#define __init
#define BUG_ON(x) assert(!(x))
#define BUG abort
#define printk(...) fprintf(stderr,__VA_ARGS__)
#define scnprintf snprintf
#define KERN_EMERG ""

#include "_prio_tree.h"

static struct node {
    struct prio_tree_node node;
    struct node *next;
} *nodes = NULL;


/*
 * Dummy definitions for VMA accesses
 */

#define PAGE_SHIFT 1

struct vm_area_struct {
    unsigned long vm_pgoff,vm_end,vm_start;
    union {
	struct prio_tree_node prio_tree_node;
    } shared;
};

#define PRIO_TREE_DEBUG
#include "_prio_tree.c"


static struct prio_tree_root tree;


static char buf[BUF_SIZE];


static void check_tree(void)
{
    if (!prio_tree_validate(&tree))
	return;
    fprintf(stderr,"BEFORE: %s\n",buf);
    prio_tree_string(&tree,buf,sizeof(buf));
    fprintf(stderr,"AFTER: %s\n",buf);
    abort();
}


static struct node insert(struct node *node,int from,int to)
{
    const struct prio_tree_node *found;

    node->node.start = from;
    node->node.last = to;
    node->next = nodes;
    nodes = node;
    prio_tree_string(&tree,buf,sizeof(buf));
    found = prio_tree_insert(&tree,&node->node);
    if (found != &node->node) {
	fprintf(stderr,"duplicate node\n");
	exit(1);
    }
    check_tree();
    return *node;
}


static void __attribute__((used)) remov(struct node *node)
{
    struct node **walk;

    prio_tree_string(&tree,buf,sizeof(buf));
    prio_tree_remove(&tree,&node->node);
    check_tree();
    for (walk = &nodes; *walk != node; walk = &(*walk)->next);
    *walk = (*walk)->next;
}


static void __attribute__((used)) dump(void)
{
    prio_tree_string(&tree,buf,sizeof(buf));
    fprintf(stderr,"DUMP: %s\n",buf);
    abort();
}


#define ins(id,start,last) struct node id = insert(&id,start,last);
#define rem(id)	 remov(&id);


int main(void)
{
    char snap_buf[BUF_SIZE];

    INIT_PRIO_TREE_ROOT(&tree);
    prio_tree_init();
#include SCRIPT_FILE
    prio_tree_string(&tree,snap_buf,sizeof(snap_buf));
    while (nodes) {
	prio_tree_string(&tree,buf,sizeof(buf));
	prio_tree_remove(&tree,&nodes->node);
	check_tree();
	nodes = nodes->next;
    }
    execlp("ptree","ptree",snap_buf,NULL);
    perror("ptree");
    exit(1);

    /* make gcc happy */
    (void) prio_tree_first;
    (void) prio_tree_next;
}
